/*
** File: admxrc6tl.h  
** Project: ADB3 core driver
** Purpose: Definitions specific to ADM-XRC-6TL.
**
** (C) Copyright Alpha Data 2009-2010
*/

#ifndef _ADATA_CORE_ADMXRC6TL_H
#define _ADATA_CORE_ADMXRC6TL_H

#include <df.h>
#include "adb3.h"
#include "admxrc6tx.h"

/* Always LITTLE ENDIAN when stored in hardware */
#pragma pack(1)
typedef struct _VpdAdmxrc6tlRev0 {
	uint16_t version;       /* Revision of this structure (0) */
	uint16_t length;        /* Length of this structure including checksum */
	uint32_t serial;
	uint32_t modifications;
	uint32_t lclkRef;
	struct {
		uint16_t speed;       /* In units of 100 kHz */
		uint8_t	 order;       /* Log base 2 of size in words */
    uint8_t  _reserved1;
    uint16_t link;        /* Byte offset in VPD of extended info */
    uint16_t _reserved2;
	} sdram[4];
  struct {
    uint8_t  stepping[8];
    uint8_t  speedGrade[4];
    uint16_t device;
    uint8_t  tempGrade;
    uint8_t  _reserved1;
  } fpga;
	uint8_t  pcbRev;
	uint8_t  cpldRev;
  uint8_t  _reserved1;
  uint8_t  checksum;      /* This MUST ALWAYS be the last member in the struct */
} VpdAdmxrc6tlRev0;
#pragma pack()

/* Always LITTLE ENDIAN when stored in hardware */
#pragma pack(1)
typedef struct _VpdAdmxrc6tlRev1 {
	uint16_t version;       /* Revision of this structure (0) */
	uint16_t length;        /* Length of this structure including checksum */
	uint32_t serial;
	uint32_t modifications;
	uint32_t lclkRef;
	struct {
		uint16_t speed;       /* In units of 100 kHz */
		uint8_t	 order;       /* Log base 2 of size in words */
    uint8_t  _reserved1;
    uint16_t link;        /* Byte offset in VPD of extended info */
    uint16_t _reserved2;
	} sdram[4];
  struct {
    uint8_t  stepping[8];
    uint8_t  speedGrade[4];
    uint16_t device;
    uint8_t  tempGrade;
    uint8_t  _reserved1;
  } fpga;
	uint8_t  pcbRev;
	uint8_t  cpldRev;
  uint8_t  _reserved1[2];
  uint32_t mgtRef;
  uint8_t  checksum;      /* This MUST ALWAYS be the last member in the struct */
} VpdAdmxrc6tlRev1;
#pragma pack()

/* Values for VpdAdmxrc6tlRev*::modifications */
#define ADMXRC6TL_VPD_MOD_NOTFITTED_XMC_PN6 (0x1U << 0U) /* XMC P16 connector NOT fitted */
#define ADMXRC6TL_VPD_MOD_NOTFITTED_XMC_PN4 (0x1U << 1U) /* PMC Pn4 connector NOT fitted */

/* Values for VpdAdmxrc6tlRev*::fpga::tempGrade */
#define ADMXRC6TL_VPD_TEMPGRADE_COMMERCIAL  ADMXRC6Tx_VPD_TEMPGRADE_COMMERCIAL
#define ADMXRC6TL_VPD_TEMPGRADE_INDUSTRIAL  ADMXRC6Tx_VPD_TEMPGRADE_INDUSTRIAL
#define ADMXRC6TL_VPD_TEMPGRADE_EXTENDED    ADMXRC6Tx_VPD_TEMPGRADE_EXTENDED   /* Added in ADM-XRC-6TL VPD spec. v1.3 */

/* Fields for ModelRegsAdmxrc6tx::intCtl */
#define ADMXRC6TL_V_INTCTL_LEVELBITS (ADMXRC6Tx_V_INTCTL_SYSLEVEL)

#endif


